@@ -30,11 +30,17 @@ sub init {
 	print Dumper([@_]);
 }
 
-sub arg {
-	print "arg: ";
+sub set_arg {
+	print "set_arg: ";
 	print Dumper([@_]);
 }
 
+sub get_arg {
+	print "get_arg: ";
+	print Dumper([@_]);
+	return "$_[1]-result";
+}
+
 sub class_init {
 	print "class_init: ";
 	print Dumper([@_]);
@@ -56,6 +62,7 @@ $b->signal_connect("clicked", sub { destroy $w });
 
 $b->set("Foo::blorp", 'fibble');
 $b->set("Foo::bletch", 'fabble');
+print "|",$b->get("Foo::blorp"),"|\n";
 
 $w->add($b);
 
